@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2004, 2005 Robert Lougher <rob@lougher.demon.co.uk>.
+ * Copyright (C) 2003, 2004, 2005, 2006 Robert Lougher <rob@lougher.demon.co.uk>.
  *
  * This file is part of JamVM.
  *
@@ -58,13 +58,13 @@
 
 /* Bits used within the chunk header (see also alloc.h) */
 #define ALLOC_BIT               1
-#define REFERENCE_BIT           4
+#define SPECIAL_BIT             4
 
 /* Macros for getting values from the chunk header */
 #define HEADER(ptr)             *((uintptr_t*)ptr)
-#define HDR_SIZE(hdr)           (hdr & ~(ALLOC_BIT|FLC_BIT|REFERENCE_BIT))
+#define HDR_SIZE(hdr)           (hdr & ~(ALLOC_BIT|FLC_BIT|SPECIAL_BIT))
 #define HDR_ALLOCED(hdr)        (hdr & ALLOC_BIT)
-#define HDR_REF_OBJ(hdr)        (hdr & REFERENCE_BIT)
+#define HDR_SPECIAL_OBJ(hdr)    (hdr & SPECIAL_BIT)
 
 /* 1 word header format
   31                                       210
@@ -73,7 +73,7 @@
    -------------------------------------------
                                              ^ alloc bit
                                             ^ flc bit
-                                           ^ reference bit
+                                           ^ special bit
 */
 
 static int verbosegc;
@@ -144,6 +144,7 @@ extern int ref_referent_offset;
 extern int ref_queue_offset;
 extern int finalize_mtbl_idx;
 extern int enqueue_mtbl_idx;
+extern int ldr_vmdata_offset;
 
 /* The possible ways in which a reference may be marked in
    the mark bit array */
@@ -258,7 +259,8 @@ void initialiseAlloc(unsigned long min, unsigned long max, int verbose) {
 }
 
 extern void markInternedStrings();
-extern void markClasses();
+extern void markBootClasses();
+extern void markLoaderClasses(Object *loader, int mark, int mark_soft_refs);
 extern void markJNIGlobalRefs();
 extern void scanThreads();
 void markChildren(Object *ob, int mark, int mark_soft_refs);
@@ -270,7 +272,7 @@ static void doMark(Thread *self, int mark_soft_refs) {
     clearMarkBits();
 
     if(oom) MARK(oom, HARD_MARK);
-    markClasses();
+    markBootClasses();
     markInternedStrings();
     markJNIGlobalRefs();
     scanThreads();
@@ -362,7 +364,6 @@ static void doMark(Thread *self, int mark_soft_refs) {
          if(element && !IS_MARKED(element)) element = NULL
 
     ITERATE_OBJECT_LIST(reference, CLEAR_UNMARKED);
-
 }
 
 static uintptr_t doSweep(Thread *self) {
@@ -398,6 +399,21 @@ static uintptr_t doSweep(Thread *self) {
             freed += size;
             unmarked++;
 
+            if(HDR_SPECIAL_OBJ(hdr) && ob->class != NULL) {
+                if(IS_CLASS(ob)) {
+                    if(verbosegc) {
+                        ClassBlock *cb = CLASS_CB(ob);
+                        if(!IS_CLASS_DUP(cb))
+                            printf("<GC: Unloading class %s>\n", cb->name);
+                    }
+                    freeClassData((Class*)ob);
+                } else
+                    if(IS_CLASS_LOADER(CLASS_CB(ob->class))) {
+                        TRACE_GC(("FREE: Freeing class loader object\n"));
+                        freeClassLoaderData(ob);
+                    }
+            }
+
             TRACE_GC(("FREE: Freeing ob @%p class %s - start of block\n", ob,
                                     ob->class ? CLASS_CB(ob->class)->name : "?"));
         }
@@ -407,7 +423,7 @@ static uintptr_t doSweep(Thread *self) {
         curr = (Chunk *) ptr;
 
         /* Clear any set bits within the header */
-        curr->header &= ~(ALLOC_BIT|FLC_BIT|REFERENCE_BIT);
+        curr->header &= ~(ALLOC_BIT|FLC_BIT|SPECIAL_BIT);
 
         /* Scan the next chunks - while they are
            free, merge them onto the first free
@@ -430,6 +446,21 @@ static uintptr_t doSweep(Thread *self) {
                 freed += size;
                 unmarked++;
 
+                if(HDR_SPECIAL_OBJ(hdr) && ob->class != NULL) {
+                    if(IS_CLASS(ob)) {
+                        if(verbosegc) {
+                            ClassBlock *cb = CLASS_CB(ob);
+                            if(!IS_CLASS_DUP(cb))
+                                printf("<GC: Unloading class %s>\n", cb->name);
+                        }
+                        freeClassData((Class*)ob);
+                    } else
+                        if(IS_CLASS_LOADER(CLASS_CB(ob->class))) {
+                            TRACE_GC(("FREE: Freeing class loader object\n"));
+                            freeClassLoaderData(ob);
+                        }
+                }
+
                 TRACE_GC(("FREE: Freeing object @%p class %s - merging onto block @%p\n",
                                        ob, ob->class ? CLASS_CB(ob->class)->name : "?", curr));
 
@@ -457,34 +488,37 @@ static uintptr_t doSweep(Thread *self) {
 marked:
         marked++;
 
-        if(HDR_REF_OBJ(hdr)) {
-            Object *referent = (Object *)INST_DATA(ob)[ref_referent_offset];
+        if(HDR_SPECIAL_OBJ(hdr) && ob->class != NULL) {
+            ClassBlock *cb = CLASS_CB(ob->class);
 
-            if(referent != NULL) {
-                int ref_mark = IS_MARKED(referent);
-                ClassBlock *cb = CLASS_CB(ob->class);
+            if(IS_REFERENCE(cb)) {
+                Object *referent = (Object *)INST_DATA(ob)[ref_referent_offset];
 
-                TRACE_GC(("FREE: found Reference Object @%p class %s flags %d referent %x mark %d\n",
-                          ob, cb->name, cb->flags, referent, ref_mark));
+                if(referent != NULL) {
+                    int ref_mark = IS_MARKED(referent);
 
-                if(IS_PHANTOM_REFERENCE(cb)) {
-                    if(ref_mark != PHANTOM_MARK)
-                        goto out;
-                } else {
-                    if(ref_mark == HARD_MARK)
-                        goto out;
+                    TRACE_GC(("FREE: found Reference Object @%p class %s flags %d referent %x mark %d\n",
+                              ob, cb->name, cb->flags, referent, ref_mark));
 
-                    TRACE_GC("Clearing the referent field.\n");
-                    INST_DATA(ob)[ref_referent_offset] = 0;
-                    cleared++;
-                }
+                    if(IS_PHANTOM_REFERENCE(cb)) {
+                        if(ref_mark != PHANTOM_MARK)
+                            goto out;
+                    } else {
+                        if(ref_mark == HARD_MARK)
+                            goto out;
 
-                /* If the reference has a queue, add it to the list for enqueuing
-                   by the Reference Handler thread. */
+                        TRACE_GC(("FREE: Clearing the referent field.\n"));
+                        INST_DATA(ob)[ref_referent_offset] = 0;
+                        cleared++;
+                    }
 
-                if((Object *)INST_DATA(ob)[ref_queue_offset] != NULL) {
-                    TRACE_GC("Adding to list for enqueuing.\n");
-                    ADD_TO_OBJECT_LIST(reference, ob);
+                    /* If the reference has a queue, add it to the list for enqueuing
+                       by the Reference Handler thread. */
+
+                    if((Object *)INST_DATA(ob)[ref_queue_offset] != NULL) {
+                        TRACE_GC(("FREE: Adding to list for enqueuing.\n"));
+                        ADD_TO_OBJECT_LIST(reference, ob);
+                    }
                 }
             }
         }
@@ -892,25 +926,31 @@ got_it:
             - globals
 */
 
-void markClassStatics(Class *class, int mark_soft_refs) {
+void markClassData(Class *class, int mark, int mark_soft_refs) {
     ClassBlock *cb = CLASS_CB(class);
     FieldBlock *fb = cb->fields;
     int i;
 
-    /* Class has not linked and so is not in-use (and
-       statics have not been initialised) */
-    if(cb->state < CLASS_LINKED)
-        return;
+    TRACE_GC(("Marking class %s\n", cb->name));
+
+    if(cb->class_loader != NULL && mark > IS_MARKED(cb->class_loader))
+        markChildren(cb->class_loader, mark, mark_soft_refs);
 
     TRACE_GC(("Marking static fields for class %s\n", cb->name));
 
+    /* Static fields are initialised to default values during
+       preparation (done in the link phase).  Therefore, don't
+       scan if the class hasn't been linked */
+    if(cb->state < CLASS_LINKED)
+        return;
+
     for(i = 0; i < cb->fields_count; i++, fb++)
         if((fb->access_flags & ACC_STATIC) &&
                     ((*fb->type == 'L') || (*fb->type == '['))) {
             Object *ob = (Object *)fb->static_value;
             TRACE_GC(("Field %s %s object @%p\n", fb->name, fb->type, ob));
-            if(ob != NULL && !IS_HARD_MARKED(ob))
-                markChildren(ob, HARD_MARK, mark_soft_refs);
+            if(ob != NULL && mark > IS_MARKED(ob))
+                markChildren(ob, mark, mark_soft_refs);
         }
 }
 
@@ -919,10 +959,15 @@ void scanThread(Thread *thread) {
     Frame *frame = ee->last_frame;
     uintptr_t *end, *slot;
 
-    TRACE_GC(("Scanning stacks for thread 0x%x\n", thread));
+    TRACE_GC(("Scanning stacks for thread 0x%x id %d\n", thread, thread->id));
 
     MARK(ee->thread, HARD_MARK);
 
+    /* If there's a pending exception raised
+       on this thread mark it */
+    if(ee->exception)
+        MARK(ee->exception, HARD_MARK);
+
     slot = (uintptr_t*)getStackTop(thread);
     end = (uintptr_t*)getStackBase(thread);
 
@@ -939,6 +984,10 @@ void scanThread(Thread *thread) {
         if(frame->mb != NULL) {
             TRACE_GC(("Scanning %s.%s\n", CLASS_CB(frame->mb->class)->name, frame->mb->name));
             TRACE_GC(("lvars @%p ostack @%p\n", frame->lvars, frame->ostack));
+
+            /* Mark the method's defining class.  This should always
+               be reachable otherwise, but mark to be safe */
+            MARK(frame->mb->class, HARD_MARK);
         }
 
         end = frame->ostack;
@@ -955,89 +1004,103 @@ void scanThread(Thread *thread) {
     }
 }
 
-void markObject(Object *object) {
-    if(IS_OBJECT(object))
+void markObject(Object *object, int mark, int mark_soft_refs) {
+    if(object != NULL && mark > IS_MARKED(object))
+        markChildren(object, mark, mark_soft_refs);
+}
+
+void markRoot(Object *object) {
+    if(object != NULL)
         MARK(object, HARD_MARK);
 }
 
 void markChildren(Object *ob, int mark, int mark_soft_refs) {
+    Class *class = ob->class;
+    ClassBlock *cb = CLASS_CB(class);
 
     SET_MARK(ob, mark);
 
-    if(ob->class == NULL)
+    if(class == NULL)
         return;
- 
-    if(IS_CLASS(ob)) {
-        TRACE_GC(("Found class object @%p name is %s\n", ob, CLASS_CB(ob)->name));
-        markClassStatics((Class*)ob, mark_soft_refs);
-    } else {
-        Class *class = ob->class;
-        ClassBlock *cb = CLASS_CB(class);
 
-        if(cb->name[0] == '[') {
-            if((cb->name[1] == 'L') || (cb->name[1] == '[')) {
-                Object **body = ARRAY_DATA(ob);
-                int len = ARRAY_LEN(ob);
-                int i;
-                TRACE_GC(("Scanning Array object @%p class is %s len is %d\n", ob, cb->name, len));
+    if(mark > IS_MARKED(class))
+        markChildren((Object*)class, mark, mark_soft_refs);
 
-                for(i = 0; i < len; i++) {
-                    Object *ob = *body++;
-                    TRACE_GC(("Object at index %d is @%p\n", i, ob));
+    if(cb->name[0] == '[') {
+        if((cb->name[1] == 'L') || (cb->name[1] == '[')) {
+            Object **body = ARRAY_DATA(ob);
+            int len = ARRAY_LEN(ob);
+            int i;
+            TRACE_GC(("Scanning Array object @%p class is %s len is %d\n", ob, cb->name, len));
 
-                    if(ob != NULL && mark > IS_MARKED(ob))
-                        markChildren(ob, mark, mark_soft_refs);
-                }
-            } else {
-                TRACE_GC(("Array object @%p class is %s  - Not Scanning...\n", ob, cb->name));
+            for(i = 0; i < len; i++) {
+                Object *ob = *body++;
+                TRACE_GC(("Object at index %d is @%p\n", i, ob));
+
+                if(ob != NULL && mark > IS_MARKED(ob))
+                    markChildren(ob, mark, mark_soft_refs);
             }
         } else {
-            uintptr_t *body = INST_DATA(ob);
-            int i;
+            TRACE_GC(("Array object @%p class is %s  - Not Scanning...\n", ob, cb->name));
+        }
+    } else {
+        uintptr_t *body = INST_DATA(ob);
+        int i;
 
-            if(IS_REFERENCE(cb)) {
-                Object *referent = (Object *)body[ref_referent_offset];
+        if(IS_CLASS(ob)) {
+            TRACE_GC(("Found class object @%p name is %s\n", ob, CLASS_CB(ob)->name));
+            markClassData((Class*)ob, mark, mark_soft_refs);
+        } else
+            if(IS_CLASS_LOADER(cb)) {
+                TRACE_GC(("Mark found class loader object @%p class %s\n", ob, cb->name));
+                markLoaderClasses(ob, mark, mark_soft_refs);
+            } else
+                if(IS_VMTHROWABLE(cb)) {
+                    TRACE_GC(("Mark found VMThrowable object @%p\n", ob));
+                    markVMThrowable(ob, mark, mark_soft_refs);
+                } else
+                    if(IS_REFERENCE(cb)) {
+                        Object *referent = (Object *)body[ref_referent_offset];
 
-                TRACE_GC(("Mark found Reference object @%p class %s flags %d referent %p\n",
-                         ob, cb->name, cb->flags, referent));
+                        TRACE_GC(("Mark found Reference object @%p class %s flags %d referent %p\n",
+                                 ob, cb->name, cb->flags, referent));
 
-                if(!IS_WEAK_REFERENCE(cb) && referent != NULL) {
-                    int ref_mark = IS_MARKED(referent);
-                    int new_mark;
+                        if(!IS_WEAK_REFERENCE(cb) && referent != NULL) {
+                            int ref_mark = IS_MARKED(referent);
+                            int new_mark;
 
-                    if(IS_PHANTOM_REFERENCE(cb))
-                        new_mark = PHANTOM_MARK;
-                    else
-                        if(!IS_SOFT_REFERENCE(cb) || mark_soft_refs)
-                            new_mark = mark;
-                        else
-                            new_mark = 0;
+                            if(IS_PHANTOM_REFERENCE(cb))
+                                new_mark = PHANTOM_MARK;
+                            else
+                                if(!IS_SOFT_REFERENCE(cb) || mark_soft_refs)
+                                    new_mark = mark;
+                                else
+                                    new_mark = 0;
 
-                    if(new_mark > ref_mark) {
-                       TRACE_GC(("Marking referent object @%p mark %d ref_mark %d new_mark %d\n",
-                                                            referent, mark, ref_mark, new_mark));
-                        markChildren(referent, new_mark, mark_soft_refs);
+                            if(new_mark > ref_mark) {
+                                TRACE_GC(("Marking referent object @%p mark %d ref_mark %d new_mark %d\n",
+                                                                        referent, mark, ref_mark, new_mark));
+                                markChildren(referent, new_mark, mark_soft_refs);
+                            }
+                        }
                     }
-                }
-            }
 
-            TRACE_GC(("Scanning object @%p class is %s\n", ob, cb->name));
+        TRACE_GC(("Scanning object @%p class is %s\n", ob, cb->name));
 
-            /* The reference offsets table consists of a list of start and
-               end offsets corresponding to the references within the object's
-               instance data.  Scan the list, and mark all references. */
+        /* The reference offsets table consists of a list of start and
+           end offsets corresponding to the references within the object's
+           instance data.  Scan the list, and mark all references. */
 
-            for(i = 0; i < cb->refs_offsets_size; i++) {
-                int offset = cb->refs_offsets_table[i].start;
-                int end = cb->refs_offsets_table[i].end;
+        for(i = 0; i < cb->refs_offsets_size; i++) {
+            int offset = cb->refs_offsets_table[i].start;
+            int end = cb->refs_offsets_table[i].end;
 
-                for(; offset < end; offset++) {
-                    Object *ob = (Object *)body[offset];
-                    TRACE_GC(("Offset %d reference @%p\n", offset, ob));
+            for(; offset < end; offset++) {
+                Object *ob = (Object *)body[offset];
+                TRACE_GC(("Offset %d reference @%p\n", offset, ob));
 
-                    if(ob != NULL && mark > IS_MARKED(ob))
-                        markChildren(ob, mark, mark_soft_refs);
-                }
+                if(ob != NULL && mark > IS_MARKED(ob))
+                    markChildren(ob, mark, mark_soft_refs);
             }
         }
     }
@@ -1194,12 +1257,12 @@ Object *allocObject(Class *class) {
         if(IS_FINALIZED(cb))
             ADD_FINALIZED_OBJECT(ob);
 
-        /* If the object is an instance of a Reference class
+        /* If the object is an instance of a special class
            mark it by setting the bit in the chunk header */
 
-        if(IS_REFERENCE(cb)) {
+        if(IS_SPECIAL(cb)) {
             uintptr_t *hdr = (uintptr_t*)(((char*)ob)-HEADER_SIZE);
-            *hdr |= REFERENCE_BIT;
+            *hdr |= SPECIAL_BIT;
         }
 
         TRACE_ALLOC(("<ALLOC: allocated %s object @%p>\n", cb->name, ob));
@@ -1358,6 +1421,9 @@ Object *allocMultiArray(Class *array_class, int dim, intptr_t *count) {
 
 Class *allocClass() {
     Class *class = (Class*)gcMalloc(sizeof(ClassBlock)+sizeof(Class));
+    uintptr_t *hdr = (uintptr_t*)(((char*)class)-HEADER_SIZE);
+    *hdr |= SPECIAL_BIT;
+
     TRACE_ALLOC(("<ALLOC: allocated class object @%p>\n", class));
     return class; 
 }
@@ -1376,9 +1442,12 @@ Object *cloneObject(Object *ob) {
         if(IS_FINALIZED(CLASS_CB(clone->class)))
             ADD_FINALIZED_OBJECT(clone);
 
-        if(IS_REFERENCE(CLASS_CB(clone->class))) {
-            uintptr_t *hdr = (uintptr_t*)(((char*)clone)-HEADER_SIZE);
-            *hdr |= REFERENCE_BIT;
+        if(HDR_SPECIAL_OBJ(hdr)) {
+            uintptr_t *clone_hdr = (uintptr_t*)(((char*)clone)-HEADER_SIZE);
+            *clone_hdr |= SPECIAL_BIT;
+
+            if(IS_CLASS_LOADER(CLASS_CB(clone->class)))
+                INST_DATA(clone)[ldr_vmdata_offset] = 0;
         }
 
         TRACE_ALLOC(("<ALLOC: cloned object @%p clone @%p>\n", ob, clone));
